#include "I2C.h"
#include "Arduino.h"
#include "system.h" 
#include <avr/delay.h>

#define	TWD_SPEED       0x01			//value: 1 (Fastest), 2 (slower), ... N (much slower) 
#define	TWD_LONG_TIME	(TWD_SPEED*8) 
#define	TWD_SHORT_TIME	(TWD_SPEED*1) 

#define   Set_SDA_High    SDA_SET(1) 
#define   Set_SDA_Low     SDA_SET(0)
#define   Set_SCL_High    SCL_SET(1)
#define   Set_SCL_Low     SCL_SET(0)

#define SDA_High      getbit(I2C_PORTIN,SDA_PIN)!=0
#define SDA_Low       getbit(I2C_PORTIN,SDA_PIN)==0 
#define SCL_High      getbit(I2C_PORTIN,SCL_PIN)!=0 
#define SCL_Low       getbit(I2C_PORTIN,SCL_PIN)==0 


void SDA_SET(unsigned char status)
{
    if(status == 1)
    {
        cbi(I2C_DDR, SDA_PIN);
        cbi(I2C_PORT, SDA_PIN);  
    }
        else
        {
            sbi(I2C_DDR, SDA_PIN);
            cbi(I2C_PORT, SDA_PIN);
        }
  
  _delay_us(I2C_DELAY);
}
void SCL_SET(unsigned char status)
{
    if(status == 1)
    {
        cbi(I2C_DDR, SCL_PIN);
        cbi(I2C_PORT, SCL_PIN);
    }   
        else
        {
            sbi(I2C_DDR, SCL_PIN);
            cbi(I2C_PORT, SCL_PIN);
        }
     
  _delay_us(I2C_DELAY);
}
void StartCondition(void)   
{     
    unsigned char cTWtrytime=0;   
   
    while(++cTWtrytime)   
    {   
        Set_SDA_High;   
        Set_SCL_High;        // make sure two line is release  
   
        Set_SDA_Low;   
   
        if((SCL_High)&&(SDA_Low))   
            break;   
    }   
}     
void StopCondition(void)   
{    
    //unsigned char cTWtrytime=0;   
   
    Set_SDA_Low;   
    Set_SCL_High;   
   
    Set_SDA_High;   
}  
unsigned char Send_Byte(unsigned char cData)   
{   
    unsigned char ix;   
    unsigned char cAcknowledge;   
    //unsigned char cTWtrytime=0;   
   
    cAcknowledge = 0;   
   
    for(ix = 0; ix < 8; ix++)   
    {   
        Set_SCL_Low;
    
        if(cData&0x80)
		{
		  Set_SDA_High;
        }else{
          Set_SDA_Low;
        };   
        cData<<=1;
        Set_SCL_High;   
    }   
   
    Set_SCL_Low;   
    Set_SDA_High;                // release data line for acknowledge    
    Set_SCL_High;                // Send a clock for Acknowledge  
   
    if(SDA_High) cAcknowledge = 1; // No Acknowledge   
    Set_SCL_Low;                  // Finish Acknoledge  
   
    return(cAcknowledge);   
} 
unsigned char Read_Byte(void)   
{   
    unsigned char ix, j;   
    unsigned char cRetval;    
   
    cRetval = 0;   //????
    Set_SDA_High;   
   
    for(ix = 0; ix < 8; ix++)   
    {   
        Set_SCL_Low;   
        for(j = 0; j < TWD_SHORT_TIME; j++)   
        { };   
   
        Set_SCL_High;   
        while(SCL_Low)   
        { };   
   
        for(j = 0; j < TWD_SHORT_TIME; j++)   
        { };   
   
        cRetval<<=1;   
        if(SDA_High) cRetval++;   
    }   
   
    for(j = 0; j < TWD_SHORT_TIME; j++)   
    { };   
   
    Set_SCL_Low;   
    for(j = 0; j < TWD_SHORT_TIME; j++)   
    { };   
   
    return cRetval;   
}   
unsigned char I2C_ReadByte(unsigned char cDevAddr, unsigned char cReg)   
{   
    unsigned char cResult = 0;   
   
   
    // write reg offset 
    StartCondition();   
    if(Send_Byte(cDevAddr))   
    return 0;  // Write address   
    if(Send_Byte(cReg))   
    return 0;   
   
    // read data  
    StartCondition();   
    if (Send_Byte(cDevAddr | 0x01)) // Read address   
    return 0;   
    cResult = Read_Byte();   
   
    StopCondition();   
    return cResult;   
}   
unsigned char I2C_WriteByte(unsigned char cDevAddr, unsigned char cReg, unsigned char cData)   
{   
  StartCondition();   
    /// send device ID and write data    
    if(!Send_Byte(cDevAddr))   
    {   
        if(!Send_Byte(cReg))   
        {   
            if(!Send_Byte(cData))   
            {   
                StopCondition();   
                return(0);      // success
            }   
        }   
    }   
    StopCondition();   
    return(1);   
} 











